\version "2.16.0"
%%% Guile does not deal with accented letters
#(use-modules (ice-9 regex))
%%;; actually defined below, in a closure
#(define-public string-upper-case #f)
#(define accented-char-upper-case? #f)
#(define accented-char-lower-case? #f)

%%;; an accented character is seen as two characters by guile
#(let ((lower-case-accented-string "éèêëáàâäíìîïóòôöúùûüçœæ")
       (upper-case-accented-string "ÉÈÊËÁÀÂÄÍÌÎÏÓÒÔÖÚÙÛÜÇŒÆ"))
   (define (group-by-2 chars result)
      (if (or (null? chars) (null? (cdr chars)))
          (reverse! result)
          (group-by-2 (cddr chars)
                      (cons (string (car chars) (cadr chars))
                            result))))
   (let ((lower-case-accented-chars
          (group-by-2 (string->list lower-case-accented-string) (list)))
         (upper-case-accented-chars
          (group-by-2 (string->list upper-case-accented-string) (list))))
     (set! string-upper-case
           (lambda (str)
             (define (replace-chars str froms tos)
               (if (null? froms)
                   str
                   (replace-chars (regexp-substitute/global #f (car froms) str
                                                            'pre (car tos) 
'post)
                                  (cdr froms)
                                  (cdr tos))))
             (string-upcase (replace-chars str
                                           lower-case-accented-chars
                                           upper-case-accented-chars))))
     (set! accented-char-upper-case?
           (lambda (char1 char2)
             (member (string char1 char2) upper-case-accented-chars string=?)))
     (set! accented-char-lower-case?
           (lambda (char1 char2)
             (member (string char1 char2) lower-case-accented-chars 
string=?)))))


#(define-markup-command (smallCaps layout props text) (markup?)
  "Turn @code{text}, which should be a string, to small caps.
@example
\\markup \\small-caps \"Text between double quotes\"
@end example"
  (define (string-list->markup strings lower)
    (let ((final-string (string-upper-case
                         (apply string-append (reverse strings)))))
      (if lower
          (markup #:fontsize -2 final-string)
          final-string)))
  (define (make-small-caps rest-chars currents current-is-lower prev-result)
    (if (null? rest-chars)
        (make-concat-markup (reverse! (cons (string-list->markup
                                              currents current-is-lower)
                                            prev-result)))
        (let* ((ch1 (car rest-chars))
               (ch2 (and (not (null? (cdr rest-chars))) (cadr rest-chars)))
               (this-char-string (string ch1))
               (is-lower (char-lower-case? ch1))
               (next-rest-chars (cdr rest-chars)))
          (cond ((and ch2 (accented-char-lower-case? ch1 ch2))
                 (set! this-char-string (string ch1 ch2))
                 (set! is-lower #t)
                 (set! next-rest-chars (cddr rest-chars)))
                ((and ch2 (accented-char-upper-case? ch1 ch2))
                 (set! this-char-string (string ch1 ch2))
                 (set! is-lower #f)
                 (set! next-rest-chars (cddr rest-chars))))
          (if (or (and current-is-lower is-lower)
                  (and (not current-is-lower) (not is-lower)))
              (make-small-caps next-rest-chars
                               (cons this-char-string currents)
                               is-lower
                               prev-result)
              (make-small-caps next-rest-chars
                               (list this-char-string)
                               is-lower
                               (if (null? currents)
                                   prev-result
                                   (cons (string-list->markup
                                            currents current-is-lower)
                                         prev-result)))))))
  (interpret-markup layout props
    (if (string? text)
        (make-small-caps (string->list text) (list) #f (list))
        text)))

%%%%%%%%%%%%%%%%%%%%%
%%% date of copyright
#(define-markup-command (copyright layout props) ()
  (let* ((maintainer (chain-assoc-get 'header:maintainer props))
         (this-year (+ 1900 (tm:year (gmtime (current-time)))))
         (year (string->number (or (chain-assoc-get 'header:copyrightYear props)
                                   (number->string this-year)))))
    (interpret-markup layout props
     (markup "Copyright ©" 
             (if (= year this-year)
                 (format #f "~a" this-year)
                 (format #f "~a-~a" year this-year))
             maintainer))))

#(define-markup-command (today layout props) ()
  (let ((today (gmtime (current-time))))
   (interpret-markup layout props
     (format #f "~a-~a-~a"
             (+ 1900 (tm:year today))
             (1+ (tm:mon today))
             (tm:mday today)))))
%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%
%% zoome les fontes pour éviter la veuve de la TdM
%% du conducteur
%% merci Nicolas Sceaux
#(define-markup-list-command (fontsize-lines layout props increment args)
   (number? markup-list?)
   #:properties ((font-size 0)
                 (word-space 1)
                 (baseline-skip 2))
   (interpret-markup-list layout
                          (cons `((baseline-skip . ,(* baseline-skip (magstep increment)))
                                  (word-space . ,(* word-space (magstep increment)))
                                  (font-size . ,(+ font-size increment)))
                                props)
                          args))

#(define-markup-list-command (abs-fontsize-lines layout props size args)
  (number? markup-list?)
  (let* ((ref-size (ly:output-def-lookup layout 'text-font-size 12))
         (text-props (list (ly:output-def-lookup layout 'text-font-defaults)))
         (ref-word-space (chain-assoc-get 'word-space text-props 0.6))
         (ref-baseline (chain-assoc-get 'baseline-skip text-props 3))
         (magnification (/ size ref-size)))
    (interpret-markup-list layout
                           (cons `((baseline-skip . ,(* magnification ref-baseline))
                                   (word-space . ,(* magnification ref-word-space))
                                   (font-size . ,(magnification->font-size magnification)))
                                 props)
                           args)))

%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%% MARKUPS %%%%%%%%%%
%% format multiline paragraph markup with indentation
#(define-markup-list-command (paragraph layout props args) (markup-list?)
  (let ((indentation (markup #:pad-to-box (cons 0 3) (cons 0 0) #:null)))
    (interpret-markup-list layout props
       (make-justified-lines-markup-list (cons indentation args)))))

#(define-markup-command (line-width-ratio layout props width-ratio arg)
  (number? markup?)
  (interpret-markup layout props
   (markup #:override (cons 'line-width (* width-ratio
                                           (chain-assoc-get 'line-width props)))
           arg)))

#(define-markup-list-command (line-width-ratio-lines layout props width-ratio args)
  (number? markup-list?)
  (interpret-markup-list layout props
    (make-override-lines-markup-list
      (cons 'line-width (* width-ratio
                           (chain-assoc-get 'line-width props)))
      args)))

#(define-markup-list-command (with-line-width-ratio layout props width-ratio args)
  (number? markup-list?)
  (let* ((line-width (chain-assoc-get 'line-width props))
         (new-line-width (* width-ratio line-width))
         (indent (* 0.5 (- line-width new-line-width)))
         (stencils (interpret-markup-list layout
                     (cons `((line-width . ,new-line-width)) props)
                     args)))
    (interpret-markup-list layout props
      (map (lambda (stencil)
             (markup #:hspace indent #:stencil stencil))
           stencils))))

#(define-markup-command (force-line-width-ratio layout props ratio arg)
     (number? markup?)
   (let* ((new-line-width (* ratio (chain-assoc-get 'line-width props)))
          (line-stencil (interpret-markup layout props
                                  (markup #:override (cons 'line-width new-line-width)
                                          arg)))
          (gap (max 0
                    (- new-line-width
                       (interval-length (ly:stencil-extent line-stencil X))))))
     (interpret-markup layout props (markup #:concat (#:stencil line-stencil #:hspace gap)))))

#(define-markup-list-command (two-column-lines layout props col1 col2)
   (markup-list? markup-list?)
   (interpret-markup-list layout props
                          (make-column-lines-markup-list
                           (let ((result '()))
                             (let map-on-lists ((col1 col1)
                                                (col2 col2))
                               (if (and (null? col1) (null? col2))
                                   (reverse! result)
                                   (let ((line-col1 (if (null? col1) "" (car col1)))
                                         (line-col2 (if (null? col2) "" (car col2)))
                                         (rest-col1 (if (null? col1) '() (cdr col1)))
                                         (rest-col2 (if (null? col2) '() (cdr col2))))
                                     (set! result (cons
                                                   (markup #:fill-line
                                                           (#:null
                                                            #:force-line-width-ratio 0.45 line-col1
                                                            #:null
                                                            #:force-line-width-ratio 0.45 line-col2
                                                            #:null))
                                                   result))
                                     (map-on-lists rest-col1 rest-col2))))))))

#(define-markup-list-command (indented-lines layout props indent args)
  (number? markup-list?)
  (let* ((new-line-width (- (chain-assoc-get 'line-width props) indent))
         (lines (interpret-markup-list layout
                 (cons `((line-width . ,new-line-width)) props)
                 args)))
   (interpret-markup-list layout props
    (map (lambda (line)
          (markup #:hspace indent #:stencil line))
     lines))))

#(define-markup-list-command (wordwrap-center-lines layout props args)
  (markup-list?)
  (map (lambda (stencil)
        (interpret-markup layout props (markup #:fill-line (#:stencil stencil))))
   (interpret-markup-list layout props (make-wordwrap-lines-markup-list args))))

#(define-markup-list-command (centered-lines layout props args)
  (markup-list?)
  (let ((baseline-skip (chain-assoc-get 'baseline-skip props)))
    (space-lines baseline-skip
      (interpret-markup-list layout props
        (map (lambda (arg) (markup #:fill-line (arg)))
             args)))))

#(define-markup-command (wordwrap-center layout props args) (markup-list?)
  (interpret-markup layout props
   (make-column-markup
    (make-wordwrap-center-lines-markup-list args))))

#(define-markup-command (super layout props arg) (markup?)
  (ly:stencil-translate-axis
   (interpret-markup
    layout
    (cons `((font-size . ,(- (chain-assoc-get 'font-size props 0) 3))) props)
    arg)
   (* 0.25 (chain-assoc-get 'baseline-skip props))
   Y))

#(define-markup-list-command (columns paper props text) (markup-list?)
  (interpret-markup-list paper props
    (make-column-lines-markup-list text)))

#(define-markup-command (separation-line layout props width) (number?)
  (interpret-markup layout props
   (markup #:fill-line (#:draw-line (cons (/ (* 20 width) (*staff-size*)) 0)))))

#(define-markup-command (sep layout props) ()
   (interpret-markup layout props
                     (markup #:pad-around 1 #:fill-line (#:draw-line '(20 . 0)))))

#(define-markup-command (thickSep layout props) ()
   (interpret-markup layout props
                     (markup #:pad-around 3 #:fill-line
		      (#:draw-line '(40 . 0)))))
